﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using static ProjectWarmlyShip.DirectionType;
using ProjectWarmlyShip.Entities;

namespace ProjectWarmlyShip.DrawningObjects
{
    // Класс, отвечающий за прорисовку и перемещение объекта-сущности
    public class DrawningWarmlyShip : DrawningShip
    {
        /// <summary>
        /// Инициализация свойств
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bodyColor">Цвет палубы</param>
        /// <param name="additionalColor">Дополнительный цвет</param>
        /// <param name="shipPipes">Признак наличия труб</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        /// <returns>true - объект создан, false - проверка не пройдена, нельзя создать объект в этих размерах</returns>
        public DrawningWarmlyShip(int speed, double weight, Color bodyColor, Color additionalColor, bool shipPipes, bool shipFuel, int width, int height) :
        base(speed, weight, bodyColor, width, height, 200, 30)
        {
            if (EntityShip != null)
            {
                EntityShip = new EntityWarmlyShip(speed, weight, bodyColor, additionalColor, shipPipes, shipFuel);
            }
        }
        public override void DrawTransport(Graphics g)
        {
            if (EntityShip is not EntityWarmlyShip warmlyShip)
            {
                return;
            }
            Brush additionalBrush = new SolidBrush(warmlyShip.AdditionalColor);
            //трубы 
            if (warmlyShip.ShipPipes)
            {
                g.FillRectangle(additionalBrush, _startPosX + 180, _startPosY + 2, 3, 12);
                g.FillRectangle(additionalBrush, _startPosX + 185, _startPosY + 4, 5, 10);
            }
            //топливо
            if (warmlyShip.ShipFuel)
            {
                g.FillRectangle(additionalBrush, _startPosX + 12, _startPosY + 5, 27, 7);
            }
            base.DrawTransport(g);
        }
    }
}

